use leptos::*;
use leptos_router::*;
use crate::app::models::Post;
use crate::app::components::Link;

#[server]
pub async fn get_posts() -> Result<Vec<Post>, ServerFnError> {
    let posts = Post::get_all("posts")
        .map_err(|e| ServerFnError::ServerError(e.to_string()))?;

    Ok(posts)
}

#[server]
pub async fn get_post(
    slug: String
) -> Result<Post, ServerFnError> {
    let posts = Post::get_all("posts")
        .map_err(|e| ServerFnError::ServerError(e.to_string()))?;

    let post = posts.into_iter().find(|post| post.slug == slug)
        .ok_or(ServerFnError::ServerError("Post not found".to_string()))?;

    Ok(post)
}

#[component]
pub fn PostList() -> impl IntoView {
    let posts = create_resource(|| (), |_| get_posts());

    let posts_view = move || {
        posts.and_then(|posts| {
            posts.iter()
                .map(|post| view! {
                                <div>
                                    <img src={post.metadata.image_path.clone()} alt=format!("Image {}", post.metadata.title)/>

                                    <div>
                                        <h2>{post.metadata.title.clone()}</h2>
                                        <p>{post.metadata.description.clone()}</p>
                                        <span>{post.metadata.date.clone()}</span>
                                    </div>
                                </div>
                            })
                .collect_view()
        })
    };

    view! {
        <main class="posts">
            <h1>Blog</h1>

            <Suspense fallback=move || view! { <p>"Chargement des posts..."</p> }>
                <div class="posts__cards">{posts_view}</div>
            </Suspense>
        </main>
    }
}

#[component]
pub fn PostElement() -> impl IntoView {
    let params = use_params_map();
    let slug = move || params.with(|params| params.get("slug").cloned().unwrap_or_default());

    let post = create_resource(|| (), move |_| get_post(slug()));

    let post_view = move || {
        post.and_then(|post| {
            view! {
                <div inner_html={post.content.clone()}></div>
            }
        })
    };

    view! {
        <main class="post">
            <Suspense fallback=move || view! { <p>"Chargement des posts..."</p> }>
                {post_view}
            </Suspense>
        </main>
    }
}